//-----------------------------------------------------------------------------
// Title      : bsp_ub042_twi_master
//-----------------------------------------------------------------------------
// Funktion   : TWI-Kommunikation zwischen einem Master und bis zu 8 Slaves
// Schaltung  : myAVR MK3
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 20.02.2012
// Version    : 1.0
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <UB_MK3_2560/MK3_2560_TWI_MASTER.h>
#include <UB_MK3_2560/MK3_2560_LED.h>

//-----------------------------------------------------------------------------
// Hinweis : 
// - Quick-Jumper Port-L muss gesetzt sein
// - Diese Software initialisiert die CPU als TWI-Master
// - TWI-Verbindung zum Slave-1 muss vorhanden sein
// - die LEDs an Port-L wechseln dauernd
//   zwischen 0x55 und 0xAA bis der Master
//   den Slave-1 gefunden hat
// - dann wird zyklisch ein Byte-Wert von Adr.00
//   vom Slave angefordert und an den LEDs
//   angezeigt
// - bei einem Fehler wird wieder der Slave gesucht usw.
//-----------------------------------------------------------------------------

//----------------------------------------------------
void searchSlave(void) {
  unsigned int n;
  TWI_SEND_ID_t check;
  char ready=0;
  
  while(ready==0) {
    // versucht den Slave zu finden
    
    for(n=0;n<50000;n++) {
      // kleine Pause
      MK3_LED_ZAHL(0x55);
    }
    
    check=MK3_TWIM_READ8_SLAVE(TWI_Slave1,0x00);
    if(check==TWI_NO_FAILURE) {
      // slave wurde gefunden
      ready=1;
    }
  
    for(n=0;n<50000;n++) {
      // kleine Pause
     MK3_LED_ZAHL(0xAA);
    }
  }
} 

//----------------------------------------------------
void readSlave(void) {
  unsigned int n;
  TWI_SEND_ID_t check;
  char error=0;
  unsigned char wert=0;
  
  while(error==0) {
    // permanente Abfrage eines Bytewertes
    // bis ein TWI-Fehler erkannt wird
    
    // Abfrage eines Bytewertes an Adresse 00h
    check=MK3_TWIM_READ8_SLAVE(TWI_Slave1,0x00);
    if(check==TWI_NO_FAILURE) {
      // auslesen erfolgreich
      wert=MK3_TWIM_RD8_DATA();
    }  
    else {
      // fehler beim auslesen
      error=1;
    }
    
    // Anzeige der ausgelesenen Zahl
    for(n=0;n<50000;n++) {
      // kleine Pause
      MK3_LED_ZAHL(wert);
    }      
  }  
}

//-----------------------------------------------------------------------------
// MAIN 
//-----------------------------------------------------------------------------
int main (void)
{
    char mode=0;
    
    // Ini der LEDs
	MK3_INIT_LED();
	
	// init der CPU als TWI-MASTER
	MK3_INIT_TWI_MASTER();
	
	while (true)
	{
		if(mode==0) {
		  // hier wird der Slave-1 gesucht
		  searchSlave();
		  // slave wurde gefunden
		  mode=1;
		}
		else {
		  // zyklische abfrage vom Slave
		  readSlave();
		  // fehler in der Abfrage
		  mode=0;
		}
	}
	return 0;
}
//-----------------------------------------------------------------------------






