//-----------------------------------------------------------------------------
// Title      : bsp_ub042_twi_slave
//-----------------------------------------------------------------------------
// Funktion   : TWI-Kommunikation zwischen einem Master und bis zu 8 Slaves
// Schaltung  : myAVR MK3
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 20.02.2011
// Version    : 1.0
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <UB_MK3_2560/MK3_2560_TWI_SLAVE.h>
#include <UB_MK3_2560/MK3_2560_LED.h>
#include <UB_MK3_2560/MK3_2560_ADC.h>

//-----------------------------------------------------------------------------
// Hinweis : 
// - Quick-Jumper Port-L muss gesetzt sein
// - Ein Poti muss an F0 angeschlossen sein
// - Diese Software initialisiert die CPU als TWI Slave-1
// - TWI-Verbindung zum Master muss vorhanden sein
// - die LEDs an Port-L wechseln dauernd
//   zwischen 0x0F und 0xF0 bis der Master
//   das erste Kommando an den Slave-1 gesendet hat
// - dann wird der Poti zyklisch ausgelesen
//   und im RAM an Adr.00h gespeichert
//   zustzlich wird der wert an den LEDs
//   ausgegeben
// - Der Master kann diesen Wert auslesen
// - Bei einem TWI-Fehler wird wieder auf den Master
//   gewartet usw.
//-----------------------------------------------------------------------------

//----------------------------------------------------
void wait4Master(void) {
  unsigned int n;
  TWI_KOM_STATUS_t check;
  char ready=0;
  
  while(ready==0) {
    // warte bis Master was gesendet hat
    
    for(n=0;n<50000;n++) {
      // kleine Pause
      MK3_LED_ZAHL(0x0F);
    }
    
    check=MK3_TWIS_AKTIVITY();
    if((check==KOM_TWI_RD) || (check==KOM_TWI_WR)) {
      // master hat was gesendet
      ready=1;
    }
  
    for(n=0;n<50000;n++) {
      // kleine Pause
     MK3_LED_ZAHL(0xF0);
    }
  }
} 

//----------------------------------------------------
void handlePoti(void) {
  unsigned int uint;
  unsigned char wert;
  unsigned int n=0;
  char error=0;
  TWI_KOM_STATUS_t check;
  
  while(error==0) {
    // permanente Abfrage des Potis
    // bis ein TWI-Fehler erkannt wird
    
    // poti auslesen
    uint=MK3_ADC_READ_CH(ADC0);
    wert=(uint/4);
    
    // wert im RAM Adr.00 speichern
    MK3_TWIS_WR8_RAM(0x00,wert);
    
    for(n=0;n<1000;n++) {
      // kleine Pause und daten anzeigen
      MK3_LED_ZAHL(wert);
    }   
    
    // test ob ein TWI-Fehler vorliegt
    check=MK3_TWIS_AKTIVITY();
    if(check==KOM_TWI_ERR) {
      // fehler ist aufgetreten
      error=1;      
    }     
  }
}

//-----------------------------------------------------------------------------
// MAIN 
//-----------------------------------------------------------------------------
int main (void)
{
	char mode=0;
	
	MK3_INIT_LED();
	
	MK3_INIT_ADC();
	
	// init der CPU als TWI-SLAVE mit TWI-ADR Slave1
	MK3_INIT_TWI_SLAVE(TWI_Slave1);
	
	while (true)
	{
		if(mode==0) {
			// warten auf Master
			wait4Master();
			// Master hat was gesendet
			mode=1;
		}
		else {
			// Poti bearbeiten
			handlePoti();
			// fehler beim TWI
			mode=0;
		}
	}
	return 0;
}
//-----------------------------------------------------------------------------






